# meds/bots/__init__.py
#
#

""" bot base class. """

from meds.errors import ENOTIMPLEMENTED
from meds.scheduler import Scheduler
from meds.launcher import Launcher
from meds.kernel import kernel
from meds.object import Object
from meds.event import Event

class Bot(Scheduler, Launcher):

    """ base class for bot implementations. """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.channels = []

    def reply(self, event, txt):
        self._nr += 1
        if type(txt) in [list, tuple]: self.say(event.channel, ",".join(txt))
        else: self.say(event.channel, txt)

    def out(self, txt):
        """ lowlevel output function. """
        raise ENOTIMPLEMENTED

    def say(self, channel, txt):
        """ output text to channel. """
        self.out(txt)

    def announce(self, txt):
        """ announce text. """
        for channel in self.channels: self.say(channel, txt)

    def join(self, channel):
        """ join a channel. """
        pass

    def joinall(self):
        """ join all channels. """
        for channel in self.channels: self.join(channel)

    def cmnd(self, txt, bot=None):
        event = Event()
        event._bot = self
        event.origin = "user@bot"
        event.txt = txt
        for func in kernel.find(event.cmnd): func(event)
