# cli.py
#
#

""" CLI bot """

from meds.launcher import Launcher
from meds.log import YELLOW, ENDC
from meds.errors import ENOFUNC
from meds.kernel import kernel
from meds.event import Event
from meds.bots import Bot
from meds.cfg import cfg

import readline
import logging
import time
import sys

class CLI(Bot):

    """ console line interface bot. """

    def dispatch(self, event):
        thrs = kernel.dispatch(event)
        if thrs: kernel.waiter(thrs)

    def start(self):
        """ run command line command. """
        if not self.channels: self.channels.append("#meds")
        if cfg.main.shell: super().start()
        elif cfg.main.args:
            txt = " ".join(cfg.main.args)
            self.cmnd(txt)

    def stop(self):
        """ stop CLI bot. """
        self.kill("CLI")
        kernel.fleet.remove(self)

    def prompt(self):
        """ return bot prompt. """
        if cfg.main.colors: txt = "%s<%s " % (YELLOW, ENDC)
        else: txt = "< "
        return txt

    def out(self, txt):
        """ output text to stdout. """
        sys.stdout.write(txt)
        sys.stdout.write("\n")
        sys.stdout.flush()

    def event(self):
        event = Event()
        event._bot = self
        event.txt = input(self.prompt())
        event.origin = "root@shell"
        event.btype = str(type(self))
        return event

    def cmnd(self, txt, bot=None):
        event = Event()
        event._bot = self
        event.origin = "user@bot"
        event.txt = txt
        event.btype = str(type(self))
        thrs = kernel.dispatch(event)
        kernel.waiter(thrs)
