# store.py
#
#

""" read files from disk. """

from meds.utils.selector import selector, wanted, notwanted
from meds.utils.tijd import fn_time
from meds.object import Object
from meds.errors import ELOAD
from meds.utils.join import j
from meds.event import Event
from meds.cfg import cfg

import logging
import glob
import os

class Storage(Object):

    def scan(self, path, *args, **kwargs):
        p = Event(**kwargs)
        if args: prefix = args[0]
        else: prefix = ""
        if not path.endswith(os.sep): path += os.sep
        if prefix and os.path.isdir(path + prefix + os.sep): path += prefix + os.sep
        for root, dirs, files in os.walk(path, topdown=True):
            for fn in files:
                fnn = j(root, fn)
                timed = fn_time(fnn)
                if timed and p.start and timed < p.start: continue
                if timed and p.end and timed > p.end: continue
                yield fnn

    def all(self, *args, **kwargs):
        p = cfg.main.workdir
        return sorted(self.scan(p, *args, **kwargs), key=lambda x: fn_time(x))

    def selected(self, event):
        for fn in self.all(event.prefix, **event.boundaries):
            try: obj = Object().load(fn)
            except: logging.warn("fail %s" % fn) ; continue
            if "deleted" in obj and obj.deleted: continue
            #if not obj.get(event.prefix, None): continue
            if event.fields and not selector(obj, event.fields): continue
            if event.notwant and notwanted(obj, event.notwant): continue
            if event.want and not wanted(obj, event.want): continue
            yield obj
 
    def first(self, *args, **kwargs):
        fns = self.all(*args, **kwargs)
        for fn in fns:
            try: obj = Object().load(fn)
            except: logging.error("ELOAD %s" % fn) ; continue
            if "deleted" in obj and obj.deleted: continue
            if args and len(args) > 1 and args[1] != obj.get(args[0], ""): continue
            return obj

    def last(self, *args, **kwargs):
        fns = self.all(*args, **kwargs)[::-1]
        res = None
        for fn in fns[::-1]:
            try: obj = Object().load(fn)
            except: logging.error("ELOAD %s" % fn) ; continue
            if "deleted" in obj and obj.deleted: continue
            if args and len(args) > 1 and args[1] != obj.get(args[0], ""): continue
            res = obj
        return res

storage = Storage()
