# meds/utils/cli.py
#
#

""" misc. functions. """

from meds import __version__

import threading
import optparse
import logging
import sys
import os

def shutdown(): logging.warn("# shutdown")

def stop(name=""):
    logging.warn("# stopping")
    for thr in threading.enumerate():
        if str(thr).startswith("<_"): continue
        if name and name not in str(thr): continue
        if "stop" in dir(thr): thr.stop()
        elif "exit" in dir(thr): thr.exit()
        elif "cancel" in dir(thr): thr.cancel()
        else: thr.join(0.1)

def hello(descr, **kwargs):
    from meds import __version__
    from meds.log import YELLOW, ENDC
    version = kwargs.get("version", __version__)
    colors = kwargs.get("colors", "")
    try:
        if colors: print("%s%s #%s%s\n" % (YELLOW, descr, version,  ENDC))
        else: print("%s #%s\n" % (descr, version))
    except Exception as ex: print(ex) ; print('try setting PYTHONIOENCODING="utf-8" before starting this program.')
    
def make_opts(options):
    parser = optparse.OptionParser(usage='usage: %prog [options]', version=str(__version__))
    for option in options:
        type, default, dest, help = option[2:]
        if "store" in type:
            try: parser.add_option(option[0], option[1], action=type, default=default, dest=dest, help=help)
            except Exception as ex: logging.error("^ Opts/error %s option %s" % (str(ex), option)) ; continue
        else:
            try: parser.add_option(option[0], option[1], type=type, default=default, dest=dest, help=help)
            except Exception as ex: logging.error("^ Opts/error %s option %s" % (str(ex), option)) ; continue
    args = parser.parse_args()
    return args

def parse_cli(name="MEDS", version=__version__, eggs=False, workdir=""):
    from meds.log import loglevel
    from meds.cfg import cfg
    opts, args = make_opts(opts_defs)
    cfg.main.args = args
    cfg.main.update(vars(opts))
    if not cfg.main.workdir: cfg.main.workdir = os.path.expanduser(workdir or "~/.meds")
    if cfg.main.shell: hello(name, version=version, **cfg.main)
    loglevel(cfg.main.loglevel, colors=cfg.main.colors)
    if cfg.main.scan or eggs:
        for fn in os.listdir(os.getcwd()):
            if fn.endswith(".egg") and fn not in sys.path:
                logging.warn("# load %s" % fn.split()[-1])
                sys.path.insert(0, os.path.abspath(fn))
    return cfg

def list_eggs(filter="meds"):
    for f in sys.path:
        if ".egg" not in f: continue
        if filter and filter not in f: continue
        yield f

def show_eggs(filter="meds"):
    path = list(list_eggs(filter))[0]

opts_defs = [
              ('', '--debug', 'store_true', False, 'debug',  "enable debug mode."),
              ('', '--path', 'store_true', False, 'path',  "show syspath"),
              ('', '--scan', 'store_true', False, 'scan',  "scan working directory for eggs"),
              ('-a', '--all', 'store_true', False, 'all',  "load all plugins."),
              ('-b', '--background', 'store_true', False, 'background',  "switch to background mode."),
              ('-c', '--colors', 'store_true', False, 'colors',  "turn on color mode"),
              ('-d', '--workdir', 'string',  "", 'workdir',  "working directory."),
              ('-e', '--onerror', 'store_true', False, 'onerror',  "raise on error"),
              ('-i', '--init', 'string', "", 'init',  "whether to initialize plugins."),
              ('-l', '--loglevel', 'string', "error", 'loglevel',  "loglevel."),
              ('-m', '--modules', 'string', "", 'modules',  "list of modules to use."),
              ('-n', '--nowait', 'store_false', True, 'nowait', 'use blocking mode'),
              ('-o', '--owner', 'string', "", 'owner',  "userhost/JID of the bot owner"),
              ('', '--openfire', 'store_true', False, 'openfire', 'use openfire to connect to XMPP.'),
              ('-p', '--port', 'string', "10102", 'port',  "port to run HTTP server on."),
              ('-s', '--shell', 'store_true', False, 'shell',  "enable shell mode."),
              ('-v', '--verbose', 'store_true', False, 'verbose', 'use verbose mode.'),
              ('-x', '--exclude', 'string', "test", 'exclude',  "modules to exclude"),
              ('', '--skip', "string", "", "skip", "list of items to skip.")
          ]

opts_defs_sed = [
              ('-d', '--dir', 'string', "", 'dir_sed',  "directory to work with."),
              ('-l', '--loglevel', 'string', "error", 'loglevel',  "loglevel"),
          ]  

opts_defs_udp = [
              ('-p', '--port', 'string', "10102", 'port',  "port to run API server on"),
              ('-l', '--loglevel', 'string', "error", 'loglevel',  "loglevel"),
          ]

opts_defs_doctest = [
              ('-e', '--onerror', 'store_true', False, 'onerror',  "raise on error"),
              ('-v', '--verbose', 'store_true', False, 'verbose',  "use verbose"),
              ('-l', '--loglevel', 'string', "error", 'loglevel',  "loglevel"),
          ]

